from gogettr.capabilities.base import Capability
from gogettr.utils import merge
from typing import Iterator


class Live(Capability):
    def pull(self, max: int = None, lang: str = "all") -> Iterator[dict]:

        url = "/u/posts/livenow"
        n = 0

        for data in self.client.get_paginated(
            url,
            params={
                "lang": lang,
                "incl": "posts|stats|userinfo|shared|liked",
                "max": 20,
            },
            offset_step=20,
            offset_start=0,
        ):
            for post in data["data"]["list"]:
                id = post["activity"]["tgt_id"]

                # Information about posts is spread across four objects, so we merge them together here.
                post = merge(
                    post,
                    data["aux"]["post"].get(id),
                    data["aux"]["s_pst"].get(id),
                    {"uinf": data["aux"]["uinf"].get(post["activity"]["tgt_oid"])},
                )

                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield post



from typing import Any

from gogettr.api import ApiClient


class Capability:
    """Provides base functionality for the individual capabilities."""

    def __init__(self, client: ApiClient):
        self.client = client

    def pull(self, *args, **kwargs) -> Any:
        """Pull the desired data from GETTR."""
        raise NotImplementedError



from typing import Iterator, Literal

from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class UserActivity(Capability):
    def pull(
        self,
        username: str,
        max: int = None,
        until: str = None,
        type: Literal["posts", "comments", "likes"] = "posts",
    ) -> Iterator[dict]:
        """Pull the users' posts, comments, and likes from the API. Gettr groups all
        these different activities under the same API endpoint, so they are grouped
        here as well.

        :param str username: the username of the desired user
        :param int max: the maximum number of posts to pull
        :param str until: the earliest post ID to pull
        :param str type: whether to pull posts, comments, or likes"""

        assert type in ["posts", "comments", "likes"]

        url = f"/u/user/{username}/posts"
        n = 0  # Number of posts emitted

        # There is a fourth option, `f_u`, which for some users seems to return
        # all their activity. It does not seem to work on all users, however.
        if type == "posts":
            fp_setting = "f_uo"
        elif type == "comments":
            fp_setting = "f_uc"
        elif type == "likes":
            fp_setting = "f_ul"

        for data in self.client.get_paginated(
            url,
            params={
                "max": 20,
                "dir": "fwd",
                "incl": "posts|stats|userinfo|shared|liked",
                "fp": fp_setting,
            },
        ):

            for event in data["data"]["list"]:
                id = event["activity"]["tgt_id"]

                # Information about posts is spread across three objects, so we merge them together here.
                post = merge(
                    event, data["aux"]["post"].get(id), data["aux"]["s_pst"].get(id)
                )

                # Verify that we haven't passed the `until` post
                if until is not None and until > id:
                    return

                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield post



from concurrent.futures import ThreadPoolExecutor
from itertools import count, islice
from collections import deque
import logging
from typing import Iterator, Literal

from gogettr.capabilities.base import Capability
from gogettr.errors import GettrApiError
from gogettr.utils import b36decode, b36encode, extract, merge


class All(Capability):
    def pull(
        self,
        first: str = None,
        last: str = None,
        max: int = None,
        type: Literal["posts", "comments"] = "posts",
        order: Literal["up", "down"] = "up",
        workers: int = 10,
    ) -> Iterator[dict]:
        """Pulls all the posts from the API sequentially.

        :param str first: the id of the earliest post to include
        :param str last: the id of the last post to include
        :param int max: the maximum number of posts to pull
        :param str type: whether to pull posts or comments
        :order ["up" | "down"] order: whether to go from first to last (chronological)
            or last to first (reverse chronological)
        :param int workers: number of workers to run in parallel threads
        """

        assert type in ["posts", "comments"]

        n = 0  # How many posts we've emitted
        post_ids = self._post_id_generator(first, last, type, order)

        with ThreadPoolExecutor(max_workers=workers) as ex:
            # Submit initial work
            futures = deque(
                ex.submit(self._pull_post, post_id)
                for post_id in islice(post_ids, workers * 2)
            )

            while futures:
                result = futures.popleft().result()

                # Yield the result if it's valid
                if result is not None:
                    n += 1
                    yield result

                # Exit if we've hit the max number of posts
                if max is not None and n >= max:
                    return

                # Schedule more work, if available
                try:
                    futures.append(ex.submit(self._pull_post, next(post_ids)))
                except StopIteration:
                    # No more unscheduled post IDs to process
                    pass

    def _post_id_generator(
        self,
        first: str = None,
        last: str = None,
        type: Literal["posts", "comments"] = "posts",
        order: Literal["up", "down"] = "up",
    ) -> Iterator[str]:
        """Returns a generator of GETTR post IDs to pull."""

        # We remove the first character from the post IDs below because they are
        # always `p` and not part of the numbering scheme
        if order == "up":
            start_at = b36decode(first[1:]) if first is not None else 1
            end_at = b36decode(last[1:]) if last is not None else None
        else:
            if last is None:
                raise ValueError(
                    "the last post (i.e., the starting post) must be defined when"
                    "pulling posts reverse chronologically (we need to know where to start!)"
                )
            start_at = b36decode(last[1:])
            end_at = b36decode(first[1:]) if first is not None else 1

        for id in count(start_at, 1 if order == "up" else -1):
            yield ("p" if type == "posts" else "c") + b36encode(id)

            if end_at is not None and id == end_at:
                return

    def _pull_post(self, post_id: str) -> dict:
        """Attempt to pull the given post from GETTR."""

        try:
            data = self.client.get(
                f"/u/post/{post_id}",
                params={
                    "incl": "poststats|userinfo|posts|commentstats",
                },
                key="result",
            )
        except GettrApiError as e:
            logging.warning("Hit API error while pulling: %s", e)
            return
        if "txt" in data and data["data"]["txt"] == "Content Not Found":
            # Yes, this is how they do it. It's just a string.
            logging.info("Post %s not found...", post_id)
            return

        user_id = extract(data, ["data", "uid"])
        if user_id is None:
            return

        # At this point we know the post exists. Let's assemble and return it.
        post = merge(
            data["data"],
            dict(
                uinf=extract(data, ["aux", "uinf", user_id]),
                shrdpst=extract(data, ["aux", "shrdpst"]),
                s_pst=extract(data, ["aux", "s_pst"]),
                s_cmst=extract(data, ["aux", "s_cmst"]),
                post=extract(data, ["aux", "post"]),
            ),
        )

        return post



from typing import Iterator

from gogettr.capabilities.base import Capability


class Suggested(Capability):
    def pull(self, max: int = None) -> Iterator[dict]:
        """Pull the suggested users.

        :param int max: the maximum number of followers to pull"""

        # https://api.gettr.com/s/usertag/suggest?offset=0&max=16&incl=userinfo%7Cfollowings
        url = "/s/usertag/suggest"

        n = 0  # Number of users emitted

        for data in self.client.get_paginated(
            url,
            params={
                "max": 16,
                "incl": "userinfo|followings",
            },
            offset_step=16,
        ):
            for username in data["data"]["list"]:
                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield data["aux"]["uinf"][username]



from typing import Iterator

from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class Trends(Capability):
    def pull(
        self, max: int = None, until: str = None, lang: str = "en"
    ) -> Iterator[dict]:
        """Pull the current Gettr trends. This corresponds to the posts on the homepage.

        :param int max: the maximum number of posts to pull
        :param str until: the earliest post ID to pull
        :param str lang: the language of the trends (default `en`)"""

        url = "/u/posts/trends"
        n = 0

        for data in self.client.get_paginated(
            url,
            params={
                "lang": lang,
                "incl": "posts|stats|userinfo|shared|liked",
                "max": 20,
            },
        ):
            for post in data["data"]["list"]:
                id = post["activity"]["tgt_id"]

                # Information about posts is spread across four objects, so we merge them together here.
                post = merge(
                    post,
                    data["aux"]["post"].get(id),
                    data["aux"]["s_pst"].get(id),
                    {"uinf": data["aux"]["uinf"].get(post["activity"]["tgt_oid"])},
                )

                # Verify that we haven't passed the `until` post
                if until is not None and until > id:
                    return

                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield post



"""Defines GoGettr-specific errors."""

from typing import Any


class GettrApiError(RuntimeError):
    """This error is for when the GETTR API experiences an internal
    failure. Sometimes these can be resolved be retrying; sometimes
    not."""

    def __init__(self, issue: Any):
        self.issue = issue
        super().__init__()



"""Defines the PublicClient, which is the primary way to interface with the
unauthenticated GETTR API."""
# pylint: disable=W0622 # we have variables called "max", "all", etc.

from functools import wraps

from gogettr.api import ApiClient
from gogettr.capabilities import (
    all,
    hashtags,
    search,
    suggested,
    trends,
    user_activity,
    user_info,
    user_relationships,
    comments,
    registration_status,
    live,
)


class PublicClient:
    """A client for all the public GETTR methods. If the API doesn't require an
    account to pull the data, it belongs here."""

    def __init__(self):
        self.api_client = ApiClient()

    @wraps(user_activity.UserActivity.pull)
    def user_activity(self, *args, **kwargs):
        """Wrapper for "user_activity"."""
        return user_activity.UserActivity(self.api_client).pull(*args, **kwargs)

    @wraps(all.All.pull)
    def all(self, *args, **kwargs):
        """Wrapper for "all"."""
        return all.All(self.api_client).pull(*args, **kwargs)

    @wraps(user_info.UserInfo.pull)
    def user_info(self, *args, **kwargs):
        """Wrapper for "user_info"."""
        return user_info.UserInfo(self.api_client).pull(*args, **kwargs)

    @wraps(user_relationships.UserRelationships.pull)
    def user_relationships(self, *args, **kwargs):
        """Wrapper for "user_relationships"."""
        return user_relationships.UserRelationships(self.api_client).pull(
            *args, **kwargs
        )

    @wraps(trends.Trends.pull)
    def trends(self, *args, **kwargs):
        """Wrapper for "trends"."""
        return trends.Trends(self.api_client).pull(*args, **kwargs)

    @wraps(suggested.Suggested.pull)
    def suggested(self, *args, **kwargs):
        """Wrapper for "suggested"."""
        return suggested.Suggested(self.api_client).pull(*args, **kwargs)

    @wraps(hashtags.Hashtags.pull)
    def hashtags(self, *args, **kwargs):
        """Wrapper for "hashtags"."""
        return hashtags.Hashtags(self.api_client).pull(*args, **kwargs)

    @wraps(search.Search.pull)
    def search(self, *args, **kwargs):
        """Wrapper for "search"."""
        return search.Search(self.api_client).pull(*args, **kwargs)

    @wraps(comments.Comments.pull)
    def comments(self, *args, **kwargs):
        """Wrapper for "comments"."""
        return comments.Comments(self.api_client).pull(*args, **kwargs)

    @wraps(registration_status.RegistrationStatus.pull)
    def is_registered(self, *args, **kwargs):
        """Wrapper for "is_registered"."""
        return registration_status.RegistrationStatus(self.api_client).pull(
            *args, **kwargs
        )

    @wraps(live.Live.pull)
    def live(self, *args, **kwargs):
        """Wrapper for "live"."""
        return live.Live(self.api_client).pull(*args, **kwargs)



from typing import Iterator, Literal

from gogettr.capabilities.base import Capability


class UserRelationships(Capability):
    def pull(
        self,
        username: str,
        max: int = None,
        type: Literal["following", "followers"] = "following",
    ) -> Iterator[dict]:
        """Pull the given users' followers or the users a given user follows.

        :param str username: the username of the desired user
        :param int max: the maximum number of followers to pull
        :param str type: whether to pull followers or following"""

        # https://api.gettr.com/u/user/dineshdsouza/followers/?offset=0&max=10&incl=userstats|userinfo
        assert type in ["following", "followers"]

        if type == "following":
            url = f"/u/user/{username}/followings"
        elif type == "followers":
            url = f"/u/user/{username}/followers"

        n = 0  # Number of users emitted

        for data in self.client.get_paginated(
            url,
            params={
                "max": 20,  # They don't seem to limit this!
                "incl": "userstats|userinfo",
            },
            offset_step=20,
        ):

            # Check if we've run out of users to pull
            if len(data["data"]["list"]) == 0:
                return

            for username in data["data"]["list"]:
                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield data["aux"]["uinf"].get(username)



"""Defines the ApiClient class, which provides a standard interface for interacting with
the GETTR API."""

import itertools
import logging
import time
from typing import Callable, Iterator

import requests
from requests.exceptions import ReadTimeout

from gogettr.errors import GettrApiError

USER_AGENT = (
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36"
    "(KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
)


class ApiClient:
    """A standard and safe way to interact with the GETTR API. Catches errors, supports
    retries, etc."""

    def __init__(self, api_base_url: str = None):
        """Initializes the API client. Optionally takes in a base URL for the GETTR api."""
        self.api_base_url = api_base_url or "https://api.gettr.com"

    def get(
        self, url: str, params: dict = None, retries: int = 3, key: str = "result"
    ) -> dict:
        """Makes a request to the given API endpoint and returns the 'results' object.
        Supports retries. Soon will support authentication."""
        tries = 0
        errors = []  # keeps track of the errors we've encountered

        def handle_error(issue):
            logging.warning(
                "Unable to pull from API: %s. Waiting %s seconds before retrying (%s/%s)...",
                issue,
                4 ** tries,
                tries,
                retries,
            )
            time.sleep(4 ** tries)
            errors.append(issue)

        while tries < retries:
            logging.info("Requesting %s (params: %s)...", url, params)
            tries += 1

            try:
                resp = requests.get(
                    self.api_base_url + url,
                    params=params,
                    timeout=10,
                    headers={"User-Agent": USER_AGENT},
                )
            except ReadTimeout as err:
                handle_error({"timeout": err})
                continue
            except Exception as e:
                handle_error({"error": str(e)})
                continue

            logging.info("%s gave response: %s", url, resp.text)

            if resp.status_code in [429, 500, 502, 503, 504]:
                handle_error({"status_code": resp.status_code})
                continue

            logging.debug("GET %s with params %s yielded %s", url, params, resp.content)

            data = resp.json()
            if key in data:
                return data[key]

            # Couldn't find the key, so it's an error.
            errors.append(data)  # Retry but without sleep.

        raise GettrApiError(errors[-1])  # Throw with most recent error

    def get_paginated(
        self,
        *args,
        offset_param: str = "offset",
        offset_start: int = 0,
        offset_step: int = 20,
        result_count_func: Callable[[dict], int] = lambda k: len(k["data"]["list"]),
        **kwargs
    ) -> Iterator[dict]:
        """Paginates requests to the given API endpoint."""
        for i in itertools.count(start=offset_start, step=offset_step):
            params = kwargs.get("params", {})
            params[offset_param] = i
            kwargs["params"] = params
            data = self.get(*args, **kwargs)
            yield data

            # End if no more results
            if result_count_func(data) == 0:
                return



"""Defines the CLI for GoGettr, which is really just a thin wrapper on top of the
module."""
# pylint: disable=W0622 # we have variables called "max", "all", etc.
# pylint: disable=R0913 # it's ok for these functions to take many arguments

import json
import logging

import click
import pkg_resources

from gogettr import PublicClient

# Setup logging
logger = logging.getLogger()
logger.setLevel(logging.INFO)
log_hander = logging.FileHandler("gogettr.log")
logger.addHandler(log_hander)

client = PublicClient()


@click.group()
@click.version_option(version=pkg_resources.get_distribution("gogettr").version)
def cli():
    """GoGettr is an unauthenticated API client for GETTR."""


@cli.command()
@click.argument("username")
@click.option("--max", help="the maximum number of activities to pull", type=int)
@click.option("--until", help="the ID of the earliest activity to pull for the user")
@click.option(
    "--type",
    help="",
    type=click.Choice(["posts", "comments", "likes"]),
    default="posts",
)
def user(username, max: int = None, until: str = None, type: str = None):
    """Pull the posts, likes, or comments made by a user."""
    for post in client.user_activity(username, max=max, until=until, type=type):
        print(json.dumps(post))


@cli.command()
@click.option(
    "--first", help="the ID of the first post to pull", type=str, default="p1"
)
@click.option("--last", help="the ID of the last post to pull", type=str)
@click.option("--max", help="the maximum number of posts to pull", type=int)
@click.option(
    "--rev",
    help="increment reverse chronologically (i.e., from last to first)",
    is_flag=True,
)
@click.option(
    "--type",
    help="",
    type=click.Choice(["posts", "comments"]),
    default="posts",
)
@click.option(
    "--workers", help="the number of threads to run in parallel", type=int, default=10
)
def all(
    first=None,
    last=None,
    max: int = None,
    rev=False,
    type: str = None,
    workers: int = None,
):
    """Pull all posts (or comments) sequentially.

    Note that if iterating chronologically and both max and last are unset, then
    this command will run forever (as it will iterate through all post IDs to
    infinity). To prevent this, either specify a max, last post, or iterate
    reverse chronologically.

    Posts will be pulled in parallel according to the desired number of workers. Out of respect
    for GETTR's servers, avoid setting the number of workers to values over 50."""
    for post in client.all(
        first=first,
        last=last,
        max=max,
        order="down" if rev else "up",
        type=type,
        workers=workers,
    ):
        print(json.dumps(post))


@cli.command()
@click.argument("username")
def user_info(username):
    """Pull given user's information."""
    print(json.dumps(client.user_info(username)))


@cli.command()
@click.argument("username")
@click.option("--max", help="the maximum number of users to pull", type=int)
def user_followers(username, max: int = None):
    """Pull all a user's followers."""
    for user_dict in client.user_relationships(username, max=max, type="followers"):
        print(json.dumps(user_dict))


@cli.command()
@click.argument("username")
@click.option("--max", help="the maximum number of users to pull", type=int)
def user_following(username, max: int = None):
    """Pull all users a given user follows."""
    for user_dict in client.user_relationships(username, max=max, type="following"):
        print(json.dumps(user_dict))


@cli.command()
@click.option("--max", help="the maximum number of posts to pull", type=int)
@click.option("--until", help="the ID of the earliest post to pull")
def trends(max: int = None, until: str = None):
    """Pull all the trends (posts displayed on the home page)."""
    for post in client.trends(max=max, until=until):
        print(json.dumps(post))


@cli.command()
@click.option("--max", help="the maximum number of users to pull", type=int)
def suggested(max: int = None):
    """Pull the suggested users (users displayed on the home page)."""
    for user_dict in client.suggested(max=max):
        print(json.dumps(user_dict))


@cli.command()
@click.option("--max", help="the maximum number of hashtags to pull", type=int)
def hashtags(max: int = None):
    """Pull the suggested hashtags (the top suggestions are displayed on the
    front page).

    Note that while the first five or so hashtags have expanded information
    associated with them, later results do not."""
    for hashtag in client.hashtags(max=max):
        print(json.dumps(hashtag))


@cli.command()
@click.argument("query")
@click.option("--max", help="the maximum number of posts to pull", type=int)
def search(query, max: int = None):
    """Search posts for the given query.

    This is equivalent to putting the query in the GETTR search box and
    archiving all the posts that result.

    Note that GETTR will return a maximum of 500 results for a query."""
    for post in client.search(query, max=max):
        print(json.dumps(post))


@cli.command()
@click.option("--max", help="the maximum number of comments to pull", type=int)
@click.argument("post_id")
def comments(post_id: str, max: int = None):
    """Pull comments on a specific post."""
    for comment in client.comments(post_id=post_id, max=max):
        print(json.dumps(comment))


@cli.command()
@click.argument("username")
def registered(username: str):
    """Check if a username is registered."""
    r = client.is_registered(username=username)
    print(json.dumps(r))


@cli.command()
@click.option(
    "--max", help="the maximum number of livestream entries to pull", type=int
)
def live(max: int = None):
    """Pull livestream posts."""
    for post in client.live(max=max):
        print(json.dumps(post))



import logging
from typing import Iterator

from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class Search(Capability):
    def pull(self, query: str, max: int = None) -> Iterator[dict]:
        """Search for posts matching the given query.

        :param str query: the query to be passed to GETTR
        :param int max: the maximum number of posts to pull"""

        url = "/u/posts/srch/phrase"
        n = 0  # Number of posts emitted

        for data in self.client.get_paginated(
            url,
            params={
                "max": 20,
                "q": query,
            },
            offset_step=20,
        ):
            for event in data["data"]["list"]:
                id = event["activity"]["tgt_id"]
                user = event["activity"]["src_id"]

                if id not in data["aux"]["post"]:
                    # Should we check `s_pst` too? It seems we usually don't need to, but
                    # worth investigating further.
                    logging.warning("Unable to find post data for post %s", id)
                    continue

                # Information about posts is spread across three objects, so we merge them
                # together here.
                post = merge(
                    event,
                    data["aux"]["post"].get(id),
                    data["aux"]["s_pst"].get(id),
                    dict(uinf=data["aux"]["uinf"].get(user)),
                )

                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield post



from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class UserInfo(Capability):
    def pull(
        self,
        username: str,
    ) -> dict:
        """Pull the given user's information.

        :param str username: the username of the desired user"""

        url = f"/s/uinf/{username}"

        data = self.client.get(
            url,
            key="result",
        )

        return merge(data["data"], dict(aux=data["aux"]))



from gogettr.capabilities.base import Capability


class RegistrationStatus(Capability):
    def pull(
        self,
        username: str,
    ) -> bool:
        """Checks if the given username is registered on GETTR.

        :param str username: username to check"""

        url = f"/s/user/{username}/exists"

        data = self.client.get(
            url,
        )
        return dict(username=username, registered=data)



"""Defines small utility functions to be used internally in GoGettr."""

from typing import Any, Iterator


def merge(*dicts):
    """Merges the given dictionaries into a single dictionary, ignoring overlapping keys."""

    out = dict()
    for dictionary in dicts:
        for (key, val) in dictionary.items():
            out[key] = val
    return out


def extract(obj: dict, path: Iterator[str], default: Any = None):
    """Tries to get the object at `path` out of the object, returning `default`
    if not found."""
    for key in path:
        if isinstance(obj, dict) and key in obj:
            obj = obj[key]
        else:
            return default
    return obj


# Following two functions adapted from
# https://stackoverflow.com/questions/1181919/python-base-36-encoding
def b36encode(number: int) -> str:
    """Convert the number to base36."""
    alphabet, base36 = ["0123456789abcdefghijklmnopqrstuvwxyz", ""]

    while number:
        number, i = divmod(number, 36)
        base36 = alphabet[i] + base36

    return base36 or alphabet[0]


def b36decode(number: str) -> int:
    """Convert the base36 number to an integer."""
    return int(number, 36)



"""
Pull comments from a specific GETTR post.

:author: Konrad Iturbe
"""
from typing import Iterator

from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class Comments(Capability):
    def pull(self, post_id: str, max: int = None) -> Iterator[dict]:
        """
        Pull comments on a specific post.

        :param str post_id: ID of the post to pull comments from (e.g., "p2vhax")
        :param int max: maximum number of comment posts to pull
        """
        url = f"/u/post/{post_id}/comments"
        n = 0

        for data in self.client.get_paginated(
            url,
            params={
                "max": 20,
                "dir": "rev",
                "incl": "posts|stats|userinfo|shared|liked",
            },
            offset_step=20,
        ):
            for comment in data["data"]["list"]:
                if max is not None and n >= max:
                    return

                n += 1
                yield merge(
                    comment, dict(s_cmst=data["aux"]["s_cmst"][comment["_id"]])
                )  # Merge Engagement data with content



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gogettr', 'gogettr.capabilities']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0', 'requests>=2.25.1,<3.0.0']

entry_points = \
{'console_scripts': ['gogettr = gogettr.cli:cli']}

setup_kwargs = {
    'name': 'gogettr',
    'version': '0.8.0',
    'description': 'Extraction tool for GETTR, a "non-bias [sic] social network."',
    'long_description': '# GoGettr\n\nGoGettr is an API client for GETTR, a "non-bias [sic] social network." GoGettr is built and maintained by the [Stanford Internet Observatory](https://io.stanford.edu).\n\nThis tool does not currently require any authentication with GETTR; it gathers all its data through publicly accessible endpoints.\n\nCurrently, this tool can:\n\n* Pull posts made on the platform\n* Pull comments made on the platform\n* Pull all top "trending" hashtags\n* Pull all suggested users\n* Pull all "trending" posts (i.e., the posts on the home page)\n* Pull all posts and/or comments of a user on the platform\n* Pull all a user\'s followers\n* Pull all users a particular user follows\n* Pull all comments on a particular post\n* Pull profile information about particular users\n\nGoGettr is designed for academic research, open source intelligence gathering, and data archival. It pulls all of the data from the publicly accessible API.\n\n## Installation\n\nGoGettr is available on PyPI. To install it, simply run `pip install gogettr`. Provided your `pip` is setup correctly, this will make `gogettr` available both as a command and as a Python package. **Note that GoGettr requires Python 3.8 or higher.**\n\n## CLI Playbook\n\n**Pull all posts (starting at id 1, capped at 1m)**\n\n```bash\ngogettr all --max 1000000\n```\n\n**Pull all comments**\n\n```bash\ngogettr all --type comments --max 1000000\n```\n\n**Pull all posts (starting at a particular ID and moving backward through IDs)**\n\n```bash\ngogettr all --rev --last pay8d\n```\n\n**Pull all posts from a user**\n\n```bash\ngogettr user USERNAME --type posts\n```\n\n**Pull all comments from a user**\n\n```bash\ngogettr user USERNAME --type comments\n```\n\n**Pull all likes from a user**\n\n```bash\ngogettr user USERNAME --type likes\n```\n\n**Pull a user\'s information**\n\n```bash\ngogettr user-info USERNAME\n```\n\n## CLI Usage\n\n```text\nUsage: gogettr [OPTIONS] COMMAND [ARGS]...\n\n  GoGettr is an unauthenticated API client for GETTR.\n\nOptions:\n  --version  Show the version and exit.\n  --help     Show this message and exit.\n\nCommands:\n  all             Pull all posts (or comments) sequentially.\n  comments        Pull comments on a specific post.\n  hashtags        Pull the suggested hashtags (the top suggestions are...\n  live            Pull livestream posts.\n  registered      Check if a username is registered.\n  search          Search posts for the given query.\n  suggested       Pull the suggested users (users displayed on the home...\n  trends          Pull all the trends (posts displayed on the home page).\n  user            Pull the posts, likes, or comments made by a user.\n  user-followers  Pull all a user\'s followers.\n  user-following  Pull all users a given user follows.\n  user-info       Pull given user\'s information.\n```\n\n### `all`\n\n```text\nUsage: gogettr all [OPTIONS]\n\n  Pull all posts (or comments) sequentially.\n\n  Note that if iterating chronologically and both max and last are unset, then\n  this command will run forever (as it will iterate through all post IDs to\n  infinity). To prevent this, either specify a max, last post, or iterate\n  reverse chronologically.\n\n  Posts will be pulled in parallel according to the desired number of workers.\n  Out of respect for GETTR\'s servers, avoid setting the number of workers to\n  values over 50.\n\nOptions:\n  --first TEXT             the ID of the first post to pull\n  --last TEXT              the ID of the last post to pull\n  --max INTEGER            the maximum number of posts to pull\n  --rev                    increment reverse chronologically (i.e., from last\n                           to first)\n  --type [posts|comments]\n  --workers INTEGER        the number of threads to run in parallel\n  --help                   Show this message and exit.\n```\n\n### `comments`\n\n```text\nUsage: gogettr comments [OPTIONS] POST_ID\n\n  Pull comments on a specific post.\n\nOptions:\n  --max INTEGER  the maximum number of comments to pull\n  --help         Show this message and exit.\n```\n\n### `hashtags`\n\n```text\nUsage: gogettr hashtags [OPTIONS]\n\n  Pull the suggested hashtags (the top suggestions are displayed on the front\n  page).\n\n  Note that while the first five or so hashtags have expanded information\n  associated with them, later results do not.\n\nOptions:\n  --max INTEGER  the maximum number of hashtags to pull\n  --help         Show this message and exit.\n```\n\n### `registered`\n\n```text\nUsage: gogettr registered [OPTIONS] USERNAME\n\n  Check if a username is registered.\n\nOptions:\n  --help  Show this message and exit.\n```\n\n### `search`\n\n```text\nUsage: gogettr search [OPTIONS] QUERY\n\n  Search posts for the given query.\n\n  This is equivalent to putting the query in the GETTR search box and\n  archiving all the posts that result.\n\nOptions:\n  --max INTEGER  the maximum number of posts to pull\n  --help         Show this message and exit\n```\n\n### `suggested`\n\n```text\nUsage: gogettr suggested [OPTIONS]\n\n  Pull the suggested users (users displayed on the home page).\n\nOptions:\n  --max INTEGER  the maximum number of users to pull\n  --help         Show this message and exit.\n```\n\n### `trends`\n\n```text\nUsage: gogettr trends [OPTIONS]\n\n  Pull all the trends (posts displayed on the home page).\n\nOptions:\n  --max INTEGER  the maximum number of posts to pull\n  --until TEXT   the ID of the earliest post to pull\n  --help         Show this message and exit.\n```\n\n### `user`\n\n```text\nUsage: gogettr user [OPTIONS] USERNAME\n\n  Pull the posts, likes, or comments made by a user.\n\nOptions:\n  --max INTEGER                  the maximum number of activities to pull\n  --until TEXT                   the ID of the earliest activity to pull for\n                                 the user\n  --type [posts|comments|likes]\n  --help                         Show this message and exit.\n```\n\n### `user-followers`\n\n```text\nUsage: gogettr user-followers [OPTIONS] USERNAME\n\n  Pull all a user\'s followers.\n\nOptions:\n  --max INTEGER  the maximum number of users to pull\n  --help         Show this message and exit.\n```\n\n### `user-following`\n\n```text\nUsage: gogettr user-following [OPTIONS] USERNAME\n\n  Pull all users a given user follows.\n\nOptions:\n  --max INTEGER  the maximum number of users to pull\n  --help         Show this message and exit.\n```\n\n### `user-info`\n\n```text\nUsage: gogettr user-info [OPTIONS] USERNAME\n\n  Pull given user\'s information.\n\nOptions:\n  --help  Show this message and exit.\n```\n\n### `live`\n\n```text\nUsage: gogettr live [OPTIONS]\n\n  Pull livestream posts.\n\nOptions:\n  --max INTEGER  the maximum number of livestream entries to pull\n  --help         Show this message and exit.\n```\n\n## Module Usage\n\nYou can use GoGettr as a Python module. For example, here\'s how you would pull all a user\'s posts:\n\n```python\nfrom gogettr import PublicClient\nclient = PublicClient()\nposts = client.user_activity(username="support", type="posts")\n```\n\nFor more examples of using GoGettr as a module, check out the [tests directory](tests/). Note that the API surface can\'t be considered quite stable yet. In the case that Gettr changes their API, GoGettr\'s API may change to match (though with as few public-facing API changes as possible, however).\n\nGoGettr groups related API functionality into the same capabilities; for example, pulling users\' comments, posts, and likes is all done by the same function (inside `user_activity.py`), and pulling followers and following is done by the same function (inside `user_relationships.py`). That means there isn\'t perfect correspondence between the CLI surface and the API surface.\n\n## Development\n\nTo run gogettr in a development environment, you\'ll need [Poetry](https://python-poetry.org). Install the dependencies by running `poetry install`, and then you\'re all set to work on gogettr locally.\n\nTo run the tests, run `poetry run pytest`.\n\nTo access the CLI, run `poetry run gogettr`.\n\nTo package and release a new version on PyPI, simply create a new release tag on GitHub.\n\n## Contributing\n\nContributions are encouraged! For small bug fixes and minor improvements, feel free to just open a PR. For larger changes, please open an issue first so that other contributors can discuss your plan, avoid duplicated work, and ensure it aligns with the goals of the project. Be sure to also follow the [code of conduct](CODE_OF_CONDUCT.md). Thanks!\n\n## Logging\n\nWhen run in CLI mode, GoGettr will log extensive debug information to `gogettr.log` (in the working directory). This log will include every single request GoGettr makes, and every single response GoGettr receives. Because it\'s possible that GoGettr accidentally loses some information when parsing API responses, consider keeping this file around just in case.\n\n## Wishlist\n\nSupport for the following capabilities is planned:\n\n- ...nothing right now! (Got an idea? Submit an issue/PR!)\n',
    'author': 'Stanford Internet Observatory',
    'author_email': 'mccain@stanford.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/stanfordio/gogettr',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Iterator

from gogettr.capabilities.base import Capability
from gogettr.utils import merge


class Hashtags(Capability):
    def pull(self, max: int = None) -> Iterator[dict]:
        """Pull the suggested hashtags.

        :param int max: the maximum number of hashtags to pull"""

        # https://api.gettr.com/s/hashtag/suggest?max=20&offset=20
        url = "/s/hashtag/suggest"

        n = 0  # Number of hashtags emitted

        for data in self.client.get_paginated(
            url,
            params={
                "max": 20,
            },
            offset_step=20,
        ):
            for hashtag in data["data"]["list"]:
                # Verify that we haven't passed the max number of posts
                if max is not None and n >= max:
                    return

                n += 1
                yield merge(dict(hashtag=hashtag), data["aux"]["htinfo"].get(hashtag))
